# Created by blogger@prodevtips.com
require 'HashExt.rb'
require 'wx'
require 'wx_sugar'
require 'rubyscript2exe.rb'

class SugaryFrame < Wx::Frame
  def initialize(*args)
    super
    listen(:close)

    # We create the menu.
    menu_file = Wx::Menu.new()

    # We attach some options to the menu.
    menu_file.append(Wx::ID_SAVE, "&Save Quick Config\tCtrl-S", "Save Quick Config")
    menu_file.append(Wx::ID_OPEN, "&Load Quick Config\tCtrl-L", "Load Quick Config")
    menu_file.append(Wx::ID_EXIT, "&Quit\tCtrl-Q", "Quit")

    # We create the menu bar that will contain the menu and attach the menu to it.
    menu_bar = Wx::MenuBar.new()
    menu_bar.append(menu_file, "&File")
    set_menu_bar(menu_bar)
    
    # We setup some event handlers.
    evt_menu(Wx::ID_SAVE) { on_save_quick_config }
    evt_menu(Wx::ID_OPEN) { on_load_quick_config }
    evt_menu(Wx::ID_EXIT) { on_quit }

    # Rubyscript2exe needs this to be able to find the config file after we have
    # 'compiled' the app.
    @config_file = RUBYSCRIPT2EXE.exedir("config.txt")

    arrange_vertically(:padding => 5)

    # We lay out the interface and attach some event handlers to various buttons.
    arrange_grid( :cols => 2, :rows => 2, :proportion => 0, :vgap => 2, :minsize => true, :padding => 1 ) do
      
      add( Wx::TextCtrl[ :value => '', :size => [250, 32] ]) do | txt | 
        @root_txt = txt 
      end
      
      add( Wx::Button[ :label => 'Select Root Dir' ] ) do | button | 
        listen(:button, button, :on_root_btn) 
      end
      
      add( Wx::TextCtrl[ :value => '', :size => [250, 32] ]) do | txt | 
        @csv_txt = txt 
      end
      
      add( Wx::Button[ :label => 'Select CSV Dir' ] ) do | button | 
        listen(:button, button, :on_csv_btn) 
      end								
    end

    arrange_grid( :cols => 1, :rows => 2, :proportion => 0, :vgap => 2, :minsize => true, :padding => 1 ) do
      
      add( Wx::Button[ :label => 'GO!' ], :align => 'center') do | button | 
        listen(:button, button, :on_go_btn) 
      end
      
      add( Wx::TextCtrl[ :value => '', :style => Wx::TE_MULTILINE, :size => [400, 200] ], :align => 'center') do | txt | 
        @out_txt = txt 
      end
    end
  end

  # When the save command is executed from the file menu or the keyboard shortcut.
  def on_save_quick_config
    HashExt.hashToCsv({'root_txt' => @root_txt.value, 'csv_txt' => @csv_txt.value}, @config_file, ';')		
  end

  # When the load command is executed from the file menu or the keyboard shortcut.
  def on_load_quick_config
    config 	= HashExt.csvToHash(@config_file, ';')
    @root_txt.value = config['root_txt']
    @csv_txt.value = config['csv_txt']		
  end

  # Calls up the system dir chooser (not file chooser, that is another modal).
  def directory_chooser(msg)
    add(Wx::DirDialog[ :message => msg ]) do | dlg |
      if not dlg.show_modal == Wx::ID_OK
        return
      end
      return dlg.get_path()
    end
  end

  # When the 'select root dir' button is clicked
  def on_root_btn()
    @root_txt.value = directory_chooser('Choose the root directory of your PHP project')
  end

  # When the 'select csv dir' button is clicked
  def on_csv_btn()
    @csv_txt.value = directory_chooser('Choose the csv directory of your PHP project')
  end

  # When the 'GO!' button is clicked.
  # First we check if both paths have been defined, if they have we continue.
  # Next we get all the csv files that exist in the csv directory.
  # Then we get translation keys from smarty templates and php files.
  # The next step is to merge all keys to one hash.
  # If the target file list is empty we create a default english translation file
  # in the target directory and put all keys in that file.
  # If the target dir is not empty we loop through all files and update them with
  # new keys, we also remove all keys that are currently not found in the project
  # because no one likes zombies.
  def on_go_btn()
    root_folder = @root_txt.value
    csv_folder 	= @csv_txt.value
    unless(root_folder.empty? || csv_folder.empty?)
      csv_separator   = ';'
      target_files    = FileExt.getFileListR(csv_folder, 'csv')
      php_hash        = HashExt.dirGetPhpTransl(root_folder, /(_\(")(\w+)("\))/, 'php')
      tpl_hash        = HashExt.dirGetPhpTransl(root_folder, /(\{insert name="tr" key=")(\w+)("\})/, 'tpl')
      php_hash        = HashExt.hashCopyValsTo(php_hash, tpl_hash)

      if target_files.empty?
        target_file = csv_folder + "/enu.csv"
        HashExt.hashToCsv(php_hash, target_file, csv_separator)
        @out_txt.value += "Created: " + target_file + "\n"		
      else
        target_files.each{ |target_file|
          csv_hash      = HashExt.csvToHash(target_file, csv_separator)
          total_hash    = HashExt.hashIntersect(csv_hash, php_hash)
          HashExt.hashToCsv(total_hash, target_file, csv_separator)
          @out_txt.value += "Updated: " + target_file + "\n"
        }
      end
    else
      @out_txt.value += "The path fields can not be empty. \n"
    end
  end

  # When we terminate the program.
  def on_close(event)
    puts "CLOSING"
    event.skip()
  end	

  def on_quit
    close
  end	
end

# Default setup stuff.
class SugaryApp < Wx::App
  def on_init
    frame = SugaryFrame.new( nil, :title => "Web Developer Tools" )
    frame.show()
  end
end

# Execute the program loop.
SugaryApp.new.main_loop()